from datetime import datetime, timedelta

import jwt
from django.core.mail import send_mail
from docusign_esign import ApiClient, EnvelopesApi
from rest_framework.renderers import TemplateHTMLRenderer
from rest_framework.response import Response
from rest_framework.views import APIView

from api.serializers import DocumentNDASerializer
from core.settings import DOCUSIGN_PRIVATE_KEY_PATH, DOCUSIGN_INTEGRATION_KEY, DOCUSIGN_AUTH_SERVER, DOCUSIGN_API_SERVER


class IndexView(APIView):
    renderer_classes = [TemplateHTMLRenderer]
    template_name = 'index.html'

    def get(self, request):
        return Response(template_name=self.template_name)

    def post(self, request):
        serializer = DocumentNDASerializer(data=request.data)
        if serializer.is_valid():
            document = serializer.save()
            send_docusign_email(request, document)
            return Response(status=200, data=serializer.data)
        return Response({'error': 'Bad Request'}, status=400)


def gen_jwt_token(email):
    with open(DOCUSIGN_PRIVATE_KEY_PATH, 'rb') as rsa_key:
        private_key = rsa_key.read()
    integration_key = DOCUSIGN_INTEGRATION_KEY
    user_email = email

    cur_time = datetime.utcnow()
    exp_time = cur_time + timedelta(hours=1)

    payload = {
        'iss': integration_key,
        'sub': user_email,
        'aud': 'account-d.docusign.com',
        'iat': cur_time,
        'exp': exp_time,
        'scope': 'signature impersonation'
    }

    return jwt.encode(payload, private_key, algorithm='RS256')


def send_docusign_email(request, user_data):
    user = user_data
    api_client = ApiClient()
    api_client.host = DOCUSIGN_AUTH_SERVER

    jwt_token = gen_jwt_token(user.owner_email)
    api_client.set_default_header('Authorization', 'Bearer ' + jwt_token)
    base_url = DOCUSIGN_API_SERVER
    envelope_api = EnvelopesApi(api_client)
    envelope_definition = {
        'email_subject': 'Подписать документ',
        'email_blurb': 'Пожалуйста, подпишите документ.',
        'status': 'sent',
        'recipients': {
            'signers': [{
                'email': user.owner_email,
                'name': user.owner_name,
                'recipientId': '1',
                'clientUserId': user.pk,
                'tabs': {
                    'textTabs': [{
                        'tabLabel': 'Name',
                        'value': user.owner_name
                    }]
                }
            }]
        },
        'documents': [{
            'documentId': '1',
            'name': 'document.docx',
            'fileExtension': 'docx',
            'documentBase64': 'BASE64_ENCODED_DOCUMENT'
        }]
    }

    envelope_summary = envelope_api.create_envelope('1', envelope_definition=envelope_definition)

    sign_url = f'{base_url}/envelopes/{envelope_summary.envelope_id}/views/recipient'
    send_mail(
        'Подписать документ',
        f'Пожалуйста, подпишите документ по ссылке: {sign_url}',
        'отправитель@example.com',
        [user.email],
        fail_silently=False,
    )

    return {'status': 'success'}
